@@ -2,6 +2,7 @@
 
 from cmind.config import Config
 from cmind.repos import Repos
+from cmind.index import Index
 from cmind.automation import Automation
 from cmind import utils
 
@@ -11,6 +12,7 @@ import imp
 import importlib
 import pkgutil
 import inspect
+import logging
 
 cm = None
 
@@ -55,7 +57,7 @@ class CM(object):
 
         # Check if debug (raise error instead of soft error)
         self.debug = False
-        if debug or os.environ.get(self.cfg['env_debug'],'').strip().lower()=='yes':
+        if debug or os.environ.get(self.cfg['env_debug'],'').strip().lower() in ['yes','on','true']:
             self.debug = True
 
         # Explicit path to direcory with CM repositories and other internals
@@ -91,6 +93,20 @@ class CM(object):
         # Check Python version
         self.python_version = list(sys.version_info)
 
+        # Save output to json (only from CLI)
+        self.save_to_json = ''
+
+        # Logging
+        self.logger = None
+        self.log = []
+
+        # Index
+        self.index = None
+
+        self.use_index = False
+        if os.environ.get(self.cfg['env_index'],'').strip().lower() in ['yes','on','true']:
+            self.use_index = True
+
     ############################################################
     def error(self, r):
         """
@@ -131,6 +147,21 @@ class CM(object):
 
         sys.exit(r['return'])
 
+    ############################################################
+    def log(self, s):
+        """
+        Args:
+           s (string): log string
+
+        Returns:
+           None
+        """
+
+        # Force console
+        print (s)
+
+        return
+
     ############################################################
     def access(self, i, out = None):
         """
@@ -151,7 +182,7 @@ class CM(object):
 
             (ignore_inheritance) (bool): if True, ignore inheritance when searching for artifacts and automations
 
-            (out) (str): if 'out', tell automations and CM to output extra information to console
+            (out) (str): if 'con', tell automations and CM to output extra information to console
 
         Returns: 
             (CM return dict):
@@ -171,6 +202,10 @@ class CM(object):
         if self.cfg['flag_debug'] in i:
             self.debug = True
 
+        # Check if log
+        if self.logger is None:
+            self.logger = logging.getLogger("cm")
+
         # Parse as command line if string or list
         if type(i) == str or type(i) == list:
             import cmind.cli
@@ -186,6 +221,10 @@ class CM(object):
 
         output = i.get('out','')
 
+        # Check if save to json
+        if 'save_to_json' in i and i['save_to_json']!='':
+            self.save_to_json = i['save_to_json']
+
         # Set self.output to the output of the very first access 
         # to print error in the end if needed
         if self.output is None:
@@ -237,6 +276,23 @@ class CM(object):
             # Set only after all initializations
             self.repos = repos
 
+        # Load index
+        if self.index is None:
+            self.index = Index(self.repos_path, self.cfg)
+
+            if self.use_index:
+                r = self.index.load()
+                if r['return']>0: return r
+
+                if not r['exists']:
+                    # First time
+                    if console:
+                        print ('Warning: CM index is used for the first time. CM will reindex all artifacts now - it may take some time ...')
+
+                    r = self.access({'action':'reindex',
+                                     'automation':'repo,55c3e27e8a140e48'})
+                    if r['return']>0: return r
+
         # Check if forced common automation
         use_common_automation = True if i.get('common',False) else False
 
@@ -260,7 +316,7 @@ class CM(object):
                     automation = r['cm_automation']
 
             # Check and make an artifact (only if artifacts are not specified)
-            if r.get('artifact_found_in_current_path', False) and artifact == '':
+            if r.get('cm_artifact','')!='' and artifact == '':
                 artifact = r['cm_artifact']
 
             if r.get('registered', False):
@@ -485,6 +541,13 @@ class CM(object):
 
         r = action_addr(i)
 
+        # Check if need to save index
+        if self.use_index and self.index.updated:
+            rx = self.index.save()
+            # Ignore output for now to continue working even if issues ...
+
+            self.index.updated=False
+
         return r
 
 ############################################################
