@@ -88,6 +88,11 @@ class CAutomation(Automation):
 
         repo_meta = r['meta']
 
+        if self.cmind.use_index:
+            ii = {'out':'con'} if console else {}
+            rx = self.reindex(ii)
+
+
         return {'return':0, 'meta':repo_meta}
 
     ############################################################
@@ -257,6 +262,10 @@ class CAutomation(Automation):
 
         r = self.cmind.repos.delete(lst, remove_all = remove_all, console = console, force = force)
 
+        if self.cmind.use_index:
+            ii = {'out':'con'} if console else {}
+            rx = self.reindex(ii)
+        
         return r
 
     ############################################################
@@ -376,6 +385,11 @@ class CAutomation(Automation):
 
         # Create repository 
         r = self.cmind.repos.init(alias = alias, uid = uid, path = path, console = console, desc=desc, prefix=prefix, only_register=repo_desc_found)
+
+        if self.cmind.use_index:
+            ii = {'out':'con'} if console else {}
+            rx = self.reindex(ii)
+        
         return r
 
     ############################################################
@@ -557,6 +571,10 @@ class CAutomation(Automation):
         repo_pack_zip.close()
         repo_pack_file.close()
 
+        if self.cmind.use_index:
+            ii = {'out':'con'} if console else {}
+            rx = self.reindex(ii)
+        
         return r_new
 
     ##############################################################################
@@ -740,7 +758,7 @@ class CAutomation(Automation):
             rr['found_in_current_path'] = r['found_in_current_path']
 
             # Load meta
-            r = utils.load_json_or_yaml(file_name = path_to_repo_desc)
+            r = utils.load_json_or_yaml(path_to_repo_desc)
             if r['return']>0: return r
 
             repo_meta = r['meta']
@@ -763,43 +781,51 @@ class CAutomation(Automation):
 
         rr['registered'] = registered
 
-        # If repository found, search for _cm.yaml or _cm.json in current directory and below
         artifact_found = False
         artifact_found_in_current_path = False
 
         if found:
-            r=utils.find_file_in_current_directory_or_above([self.cmind.cfg['file_cmeta']+'.json', 
-                                                             self.cmind.cfg['file_cmeta']+'.yaml'],
-                                                             path_to_start = path,
-                                                             reverse = True)
-            if r['return']>0: return r
-
-            artifact_found = r['found']
-            artifact_found_in_current_path = r.get('found_in_current_path', False)
-
-            rr['artifact_found'] = artifact_found
-
-            if artifact_found:
-                path_to_artifact_desc = r['path_to_file']
-                path_to_artifact = r['path']
-
-                rr['path_to_artifact_desc'] = path_to_artifact_desc
-                rr['path_to_artifact'] = path_to_artifact
-
-                rr['artifact_found_in_current_path'] = artifact_found_in_current_path
-
-                # Load meta
-                r = utils.load_json_or_yaml(file_name = path_to_artifact_desc)
+            for reverse in [False, True]:
+                r=utils.find_file_in_current_directory_or_above([self.cmind.cfg['file_cmeta']+'.json', 
+                                                                 self.cmind.cfg['file_cmeta']+'.yaml'],
+                                                                 path_to_start = path,
+                                                                 reverse = reverse)
                 if r['return']>0: return r
 
-                artifact_meta = r['meta']
+                artifact_found = r['found']
+                artifact_found_in_current_path = r.get('found_in_current_path', False)
 
-                rr['artifact_meta'] = artifact_meta
+                rr['artifact_found'] = artifact_found
 
-                rr['cm_automation'] = utils.assemble_cm_object(artifact_meta['automation_alias'],artifact_meta['automation_uid'])
+                if artifact_found:
+                    path_to_artifact_desc = r['path_to_file']
+                    path_to_artifact = r['path']
 
-                if artifact_found_in_current_path:
-                    rr['cm_artifact'] = utils.assemble_cm_object(artifact_meta['alias'],artifact_meta['uid'])
+                    rr['path_to_artifact_desc'] = path_to_artifact_desc
+                    rr['path_to_artifact'] = path_to_artifact
+
+                    rr['artifact_found_in_current_path'] = artifact_found_in_current_path
+                    rr['artifact_reverse_search'] = reverse
+
+                    # Load meta
+                    path_to_artifact_desc_without_ext = path_to_artifact_desc
+                    j = path_to_artifact_desc.rfind('.')
+                    if j>=0:
+                        path_to_artifact_desc_without_ext=path_to_artifact_desc[:j]
+
+                    r = utils.load_yaml_and_json(path_to_artifact_desc_without_ext)
+                    if r['return']>0: return r
+
+                    artifact_meta = r['meta']
+
+                    rr['artifact_meta'] = artifact_meta
+
+                    rr['cm_automation'] = utils.assemble_cm_object(artifact_meta['automation_alias'],artifact_meta['automation_uid'])
+
+                    if not reverse or artifact_found_in_current_path:
+                        rr['cm_artifact'] = utils.assemble_cm_object(artifact_meta['alias'],artifact_meta['uid'])
+
+                    break
 
         # Print
         if console:
@@ -836,9 +862,64 @@ class CAutomation(Automation):
                     print ('  Artifact alias: {}'.format(artifact_meta['alias']))
                     print ('  Artifact UID:   {}'.format(artifact_meta['uid']))
 
+        if self.cmind.use_index:
+            ii = {'out':'con'} if console else {}
+            rx = self.reindex(ii)
+
         return rr
 
 
+    ############################################################
+    def reindex(self, i):
+        """
+        Reindex all CM repositories
+
+        Args:
+            (CM input dict)
+
+            (verbose) (bool): If True, print index
+
+        Returns: 
+            (CM return dict):
+
+            * return (int): return code == 0 if no error and >0 if error
+            * (error) (str): error string if return>0
+
+        """
+
+        verbose = i.get('verbose', False)
+
+        console = i.get('out') == 'con'
+
+        if console:
+            print ('')
+            print ('Reindexing all CM artifacts. Can take some time ...')
+        
+        out = 'con' if verbose else ''
+        
+        # Clean index
+        self.cmind.index.meta = {}
+        
+        r = self.cmind.access({'action':'search',
+                               'automation':'*',
+                               'out':out,
+                               'skip_index_search':True,
+                               'force_index_add':True})
+        if r['return']>0: return r
+
+        # Save
+
+        rx = self.cmind.index.save()
+        # Ignore output for now to continue working even if issues ...
+
+        if self.cmind.use_index:
+            rx = self.cmind.index.load()
+            # Ignore output for now to continue working even if issues ...
+
+
+        return {'return':0}
+
+
 
 ##############################################################################
 def convert_ck_dir_to_cm(rpath):
