@@ -45,6 +45,8 @@ class Repos:
 
         self.full_path_to_repo_paths = ''
 
+        self.extra_info = {}
+
     ############################################################
     def load(self, init = False):
         """
@@ -128,6 +130,14 @@ class Repos:
 
                     # Set only after all initializations
                     self.lst.append(repo)
+                   
+                    repo_uid = repo.meta['uid']
+                    if repo_uid!='':
+                        self.extra_info[repo_uid]=repo
+
+                    repo_alias = repo.meta['alias']
+                    if repo_alias!='':
+                        self.extra_info[repo_alias]=repo
 
                     found = True
                     break
@@ -333,8 +343,8 @@ class Repos:
             must_update_repo_desc = True
 
 
-# GF blocked the following code because if we create a fork of mlcommons@ck for example to octoml@ck
-# and then pull octoml@ck, it attempts to rewrite .cmr.yaml with the new alias which we do not want to do!
+# GF blocked the following code because if we create a fork of mlcommons@ck for example to ctuning@mlcommons-ck
+# and then pull ctuning@mlcommons-ck, it attempts to rewrite .cmr.yaml with the new alias which we do not want to do!
 # We want to keep whatever is in .cmr.yaml to avoid ambiguities ...
 #        else:
 #            # Load meta from the repository
